StomperZ = { };

function Decents() return true end;
function WayOffs() return false end;

-- The mod parses check _G for the function, it cannot be namespaced.
ModsMaster.NoteSkin = { fnctn = 'StomperZNoteSkin' }
function StomperZNoteSkin()
	local ops = {'Cel','Metal','Flat','Robot','Vivid'};
	local mods = {}; 
	for i,v in ipairs( ops ) do
		mods[i] = v .. ( StomperZ.UseZNoteSkins and "z" or "" );
	end
	local t = OptionRowBase( "NoteSkin", ops )
	t.LoadSelections = function(self, list, pn) 
		local k = 0
		for i,v in ipairs(mods) do 
			if CheckMod(pn,v) then k = i; end 
		end 
		if k == 0 and StomperZ.UseZNoteSkins then
			for i,v in ipairs(ops) do 
				if CheckMod(pn,v) then k = i; end 
			end 
		end
		if k == 0 then k = 1 end
		list[k] = true 
	end
	t.SaveSelections = function(self, list, pn) for i,v in ipairs(list) do if v then ApplyMod(mods[i],pn+1) end end end
	return t
end	

-- Move the playfields if the given options are selected (this is called in PlayerPnOnCommand)
function StomperZPlayfieldPosition(playfield,pn)
	if not ModCustom.TargetPosition then ModCustom.TargetPosition = {1,1} end
	if ModCustom.TargetPosition[pn] == 2 then
		local val = GAMESTATE:PlayerIsUsingModifier(pn-1,"Reverse") and 25 or -45
		playfield:addy(val)
	end
end

-- Option row for custom mod
function TargetPositionOption()
	return CustomMod('Target Position','TargetPosition',{"ITG","StomperZ"})
end
ModsMaster.TargetPosition =	{ fnctn = 'TargetPositionOption' }

-- Add TargetPosition to the list of remembered modifiers. Too bad we didn't think ahead make the init a separate table we could just add to.
local OriginalInitializer = InitializeMods;
function InitializeMods()
	if GAMESTATE:GetEnv('Mods') then return end
	OriginalInitializer();
	ModCustom.TargetPosition = ModCustom.TargetPosition or {1,1}
	for pn = 1,2 do 
		if GAMESTATE:IsPlayerEnabled(pn-1) then 
			ModCustom.TargetPosition[pn] = Profile(pn).Mods.TargetPosition or ModCustom.TargetPosition[pn];
		end
	end
end


StomperZ.LifeMeterWidth = ( SCREEN_HEIGHT-64 ) * 2

function StomperZ.PlayerOptionsInit()
	local ret = PlayerOptionsInit()
	for i,v in ipairs( optionsList ) do
		if v == "LifeBar" then
			optionsList[i] = "TargetPosition";
			break
		end
	end
	return ret
end

function SurroundLife() -- This functions is called anyway, might as well use it to set up Double/SoloSingle conditons.
	if GAMESTATE:PlayerUsingBothSides() or ( GAMESTATE:GetNumPlayersEnabled() == 1 and GetPref('SoloSingle') ) then 
		for pn=1,2 do 
			if GAMESTATE:IsPlayerEnabled(pn-1) then
				local life = Screen():GetChild('LifeP'..pn)
				life:zoomx(2);
				life:x(SCREEN_CENTER_X);
			end
		end
	end
end;

local ScreenGameplayUnderlay;
local IgnoreLeaves;
local FirstLeaf;
local ActiveFrames;
local QueueFrames;
local TargetFrames;
local FirstSprite;
local FramesFound;

function StomperZ.GameplayUnderlayOn(self)
	ScreenGameplayUnderlay = self;
	IgnoreLeaves = {};
	FirstLeaf = nil;
	ActiveFrames = { { self , self:GetNumChildren() } }
	QueueFrames = { };
	TargetFrames = {};
	FirstSprite = true;
	FramesFound = 0;
	self:propagate(1);
	self:queuecommand('Capture')
end

function StomperZ.GameplayUnderlayCapture(self)

	-- sadly, we cannot simply check IgnoreLeaves[self], self has a different lua value each time but still satisfies the equality.
	for k,v in pairs( IgnoreLeaves ) do
		if k == self then return end
	end

	if IsType( self, 'ActorFrame' ) then
		local n = self:GetNumChildren();
		StomperZ.ContainsDifficultyMeter = false;
		self:playcommand("Check");
		if StomperZ.ContainsDifficultyMeter then
			if n > 0 then
				self:propagate(1)
				table.insert( QueueFrames, { self, n } )
			else
				IgnoreLeaves[self] = true;
			end
		else
			IgnoreLeaves[self] = true;
		end
	else
		self:playcommand("Check");
		if IsType( self, 'DifficultyMeter' ) then
			TargetFrames[StomperZ.ContainsDifficultyMeter] = ActiveFrames[1][1]
			FramesFound = FramesFound + 1;
		elseif FirstSprite and IsType( self, 'Sprite' ) then
			FirstSprite = false
			self:stretchto(SCREEN_LEFT,SCREEN_TOP,SCREEN_RIGHT,40)
		end
		IgnoreLeaves[self] = true;
	end	

	local n = ActiveFrames[1][2] - 1
	if n == 0 then
		table.remove( ActiveFrames, 1 )
	else
		ActiveFrames[1][2] = n
	end

	if table.getn(ActiveFrames) == 0 then
		if FramesFound == GAMESTATE:GetNumPlayersEnabled() then
			if TargetFrames[1] then
				TargetFrames[1]:y(20)
			end
			if TargetFrames[2] then
				TargetFrames[2]:y(20)
			end
		else
			ActiveFrames = QueueFrames;
			QueueFrames = { };
			ScreenGameplayUnderlay:queuecommand("Capture")
		end
	end
end

function JudgmentColor( n , alpha )
	if alpha then a = alpha else a = 1 end
	if n == 1 then return 0,.86,1,a end
	if n == 2 then return 1,  1,1,a end
	if n == 3 then return 1,.60,0,a end
	if n == 4 then return .04,1,0,a end
	if n == 5 then return .62,0,.97,a end
	if n == 6 then return 1,0,0,a end
	return 1,1,1,1
end

function StomperZ.JudgmentColorBase( n )
	local c = {}
	if n == 1 then c={0,.86,1} end
	if n == 2 then c={1,  1,1} end
	if n == 3 then c={1,.60,0} end
	if n == 4 then c={.04,1,0} end
	if n == 5 then c={.62,0,.97} end
	if n == 6 then c={1,0,0} end
	return c
end

function StomperZ.JudgmentColorLight( n , a )
	local c = StomperZ.JudgmentColorBase( n )
	for i,k in ipairs(c) do c[i] = k+(1-k)*a end
	return c[1],c[2],c[3],1
end

function StomperZ.JudgmentColorDark( n , a )
	local c = StomperZ.JudgmentColorBase( n )
	for i,k in ipairs(c) do c[i] = k+(.8-k)*a end
	return c[1],c[2],c[3],1
end	

function StomperZ.GradeFrameCommand(self)
	local a = 0.5
	if self:GetText() == "FANTASTIC"	then self:diffuse(StomperZ.JudgmentColorDark(1,a)) end
	if self:GetText() == "EXCELLENT"	then self:diffuse(StomperZ.JudgmentColorDark(2,a)) self:settext("FANTASTIC") end
	if self:GetText() == "GREAT"		then self:diffuse(StomperZ.JudgmentColorDark(3,a)) self:settext("EXCELLENT") end
	if self:GetText() == "DECENT"		then self:diffuse(StomperZ.JudgmentColorDark(4,a)) self:settext("GREAT") end
	if self:GetText() == "WAY OFF"		then self:diffuse(StomperZ.JudgmentColorDark(5,a)) self:settext("DECENT") end
	if self:GetText() == "MISS"		then self:diffuse(StomperZ.JudgmentColorDark(6,a)) end
end

local GamePrefs = {}
GamePrefs.ITG = {
	RegenComboAfterMiss =			5;
	JudgeWindowAdd =			0.001500;

	JudgeWindowSecondsMarvelous =		0.021500;
	JudgeWindowSecondsPerfect =		0.043000;
	JudgeWindowSecondsGreat =		0.102000;
	JudgeWindowSecondsGood =		0.135000;
	JudgeWindowSecondsBoo =			0.180000;
	JudgeWindowSecondsMine =		0.070000;
	JudgeWindowSecondsOK =			0.320000;
	
	GradeWeightMarvelous =	 		 2;
	GradeWeightPerfect =	 		 1;
	GradeWeightGreat =			 0;
	GradeWeightGood =			 0;
	GradeWeightBoo =			0;
	GradeWeightMiss =			0;
	GradeWeightNG =				 0;
	GradeWeightOK =				 1;
	GradeWeightHitMine =			-1;

	PercentScoreWeightMarvelous =	 	 2;
	PercentScoreWeightPerfect =		 1;
	PercentScoreWeightGreat =		 0;
	PercentScoreWeightGood =		 0;
	PercentScoreWeightBoo =			0;
	PercentScoreWeightMiss =		0;
	PercentScoreWeightNG =			 0;
	PercentScoreWeightOK =			 1;
	PercentScoreWeightHitMine =		-1;

	LifeDeltaPercentChangeMarvelous =	 0.008;
	LifeDeltaPercentChangePerfect =		 0.008;
	LifeDeltaPercentChangeGreat =		 0.004;
	LifeDeltaPercentChangeGood =		 0.000;
	LifeDeltaPercentChangeBoo =		-0.050;
	LifeDeltaPercentChangeMiss =		-0.100;
	LifeDeltaPercentChangeNG =		-0.080;
	LifeDeltaPercentChangeOK =		 0.008;
	LifeDeltaPercentChangeHitMine =	-0.050;
}

GamePrefs.StomperZ = {
	RegenComboAfterMiss =			5;
	JudgeWindowAdd =			0.001500;
	
	JudgeWindowSecondsMarvelous =		0.011000;
	JudgeWindowSecondsPerfect =		0.021500;
	JudgeWindowSecondsGreat =		0.043000;
	JudgeWindowSecondsGood =		0.102000;
	JudgeWindowSecondsBoo =		0.102000;
	JudgeWindowSecondsMine =		0.070000;
	JudgeWindowSecondsOK =			0.320000;

	GradeWeightMarvelous =	 		2;
	GradeWeightPerfect =	 		1;
	GradeWeightGreat =			0;
	GradeWeightGood =		 	0;
	GradeWeightBoo =		 	0;
	GradeWeightMiss =		 	0;
	GradeWeightNG =			 	0;
	GradeWeightOK =			 	1;
	GradeWeightHitMine =			-1;

	PercentScoreWeightMarvelous =	 	2;
	PercentScoreWeightPerfect =		1;
	PercentScoreWeightGreat =		4;
	PercentScoreWeightGood =		0;
	PercentScoreWeightBoo =		0;
	PercentScoreWeightMiss =		0;
	PercentScoreWeightNG =			0;
	PercentScoreWeightOK =			1;
	PercentScoreWeightHitMine =		-1;

	LifeDeltaPercentChangeMarvelous =	 0.008;
	LifeDeltaPercentChangePerfect =	 0.008;
	LifeDeltaPercentChangeGreat =		 0.008;
	LifeDeltaPercentChangeGood =		 0.004;
	LifeDeltaPercentChangeBoo =		 0.000;
	LifeDeltaPercentChangeMiss =		-0.100;
	LifeDeltaPercentChangeNG =		-0.080;
	LifeDeltaPercentChangeOK =		 0.008;
	LifeDeltaPercentChangeHitMine =	-0.050;
}

local function SetPreferences( t ) for p,v in pairs( t ) do SetPref( p, v ) end end
function StomperZ.SetITG() return SetPreferences( GamePrefs.ITG ) end
function StomperZ.SetStomperZ() return SetPreferences( GamePrefs.StomperZ ) end
